#include "DNN/ANN_C.h"
#include "DNN/multiann.h"

/**
   \file ANN_C.C
   \brief Implementation of the simple C interface.
   
   Implementation of the simple C interface for the Yershova and Lavalle KDTree implementation.
*/

TKDTree *CreateKDTree(int dim)
{
  int *topology;
  ANNpoint scale;
  unsigned int i;
  MultiANN *MAG;
  
  topology=new int[dim];
  scale=new double[dim];

  for(i=0;i<dim;i++)
    {
      topology[i]=1;
      scale[i]=1.0;
    }

  MAG = new MultiANN(dim,1,topology,scale);
  
  delete [] topology;
  delete [] scale;

  return((TKDTree *)MAG);
}

TKDTree *CreateKDTreeT(int dim,int *topology)
{
  if (topology==NULL)
    return(CreateKDTree(dim));
  else
    {
      ANNpoint scale;
      unsigned int i;
      MultiANN *MAG;
      
      scale=new double[dim];
      
      for(i=0;i<dim;i++)
	scale[i]=1.0;
      
      MAG = new MultiANN(dim,1,topology,scale);
      
      delete [] scale;

      return((TKDTree *)MAG);
    }
}

TKDTree *CreateKDTreeTS(int dim,int *topology,double *scale)
{
  if ((topology==NULL)&&(scale==NULL))
    return(CreateKDTree(dim));
  else
    {
      if ((topology!=NULL)&&(scale==NULL))
	return(CreateKDTreeT(dim,topology));
      else
	{
	  MultiANN *MAG;
  
	  MAG = new MultiANN(dim,1,topology,scale);
      
	  return((TKDTree *)MAG);
	}
    }
}

void AddPoint2KDTree(double *p,TKDTree *tree)
{   
  ((MultiANN *)tree)->AddPoint(p,p);
}

void QueryKDTree(double *q,int *ndx,double *d,TKDTree *tree)
{
  ((MultiANN *)tree)->NearestNeighbor(q,*ndx,*d);
}

void QueryKDTreeR(double *q,double r,int *k,
		  int **ndx,double **d,TKDTree *tree)
{
  ((MultiANN *)tree)->NearestNeighbor(q,r,k,d,ndx);
}

void DeleteKDTree(TKDTree *tree)
{
  delete ((MultiANN *)tree);
}


